/* Copyright (C) 2002 Jacob Kroon, see COPYING for details */

#ifdef _NOGGER_USE_ALSA

#include <alsa/asoundlib.h>

/* statics */
static snd_pcm_t *_pcm_handle = NULL;

/* functions */
void audio_open(int speed, int nbr_of_channels)
{
  snd_pcm_hw_params_t *hwparams;

  snd_pcm_hw_params_alloca(&hwparams);
  snd_pcm_open(&_pcm_handle, "plughw:0,0", SND_PCM_STREAM_PLAYBACK, 0);
  snd_pcm_hw_params_any(_pcm_handle, hwparams);
  snd_pcm_hw_params_set_access(_pcm_handle, hwparams,
			       SND_PCM_ACCESS_RW_INTERLEAVED);
  snd_pcm_hw_params_set_format(_pcm_handle, hwparams, SND_PCM_FORMAT_S16_LE);
  snd_pcm_hw_params_set_rate_near(_pcm_handle, hwparams, speed, 0);
  snd_pcm_hw_params_set_channels(_pcm_handle, hwparams, nbr_of_channels);
  snd_pcm_hw_params_set_periods(_pcm_handle, hwparams, 2, 0);
  snd_pcm_hw_params_set_buffer_size(_pcm_handle, hwparams, 2 << 14);
  snd_pcm_hw_params(_pcm_handle, hwparams);
  snd_pcm_hw_params_free(hwparams);
  snd_pcm_prepare(_pcm_handle);
}

void audio_close(void)
{
  snd_pcm_close(_pcm_handle);
  _pcm_handle = NULL;
}

void audio_write(char *data, int size)
{
  if (_pcm_handle != NULL)
    while (snd_pcm_writei(_pcm_handle, data, size / 4) < 0)
      snd_pcm_prepare(_pcm_handle);
}

#else
#ifdef _NOGGER_USE_OSS

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

/* defines */
#define BLOCK_SIZE		8		 /* 2^8 = 256 bytes */
#define NBR_OF_FRAGMENTS	16		 /* Increase if it's noisy(?) */

/* static variables */
static int _devdsp = -1;

/* functions */
void audio_open(int speed, int nbr_of_channels)
{
  int fragment = NBR_OF_FRAGMENTS << 16 | BLOCK_SIZE;
  int format = AFMT_S16_LE;

  if (_devdsp != -1)
    return;
  _devdsp = open("/dev/dsp", O_WRONLY);
  ioctl(_devdsp, SNDCTL_DSP_SETFRAGMENT, &fragment);
  ioctl(_devdsp, SNDCTL_DSP_SETFMT, &format);
  ioctl(_devdsp, SNDCTL_DSP_CHANNELS, &nbr_of_channels);
  ioctl(_devdsp, SNDCTL_DSP_SPEED, &speed);
}

void audio_close(void)
{
  if (_devdsp == -1)
    return;
  close(_devdsp);
  _devdsp = -1;
}

void audio_write(char *data, int size)
{
  write(_devdsp, data, size);
}

#else
#ifdef _NOGGER_USE_AO

#include <stdio.h>
#include <ao/ao.h>

/* static variables */
static ao_device *_dev = NULL;

/* functions */
void audio_open(int speed, int nbr_of_channels)
{
  ao_sample_format format;
  ao_option *options = NULL;

  if (_dev != NULL)
    return;
  ao_initialize();
  format.bits = 16;
  format.rate = speed;
  format.channels = nbr_of_channels;
  format.byte_format = AO_FMT_LITTLE;
  ao_append_option(&options, "buf_size", "256");
  _dev = ao_open_live(ao_default_driver_id(), &format, options);
}

void audio_close(void)
{
  if (_dev == NULL)
    return;
  ao_close(_dev);
  ao_shutdown();
  _dev = NULL;
}

void audio_write(char *data, int size)
{
  ao_play(_dev, data, size);
}

#endif
#endif
#endif
