/* Copyright (C) 2002 Jacob Kroon, see COPYING for details */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "nogger.h"

/* static variables */
static int _is_pressed = 0;
static GtkObject *_adjustment = NULL;
static GtkWidget *_window = NULL;

/* static functions */
static gint _pressed_cb(GtkWidget * widget, GdkEventButton * event,
			gpointer data)
{
  _is_pressed = 1;
  return FALSE;
}

static gint _released_cb(GtkWidget * widget, GdkEventButton * event,
			 gpointer data)
{
  _is_pressed = 0;
  ogg_jump_to(gtk_adjustment_get_value(GTK_ADJUSTMENT(_adjustment)));
  gui_update(NULL);
  return FALSE;
}

static void _pause_cb(GtkToggleButton * button, gpointer data)
{
  ogg_toggle_pause();
}

static void _random_cb(GtkToggleButton * button, gpointer data)
{
  ogg_toggle_random();
}

static GtkWidget *_toolbar_create(void)
{
  GtkWidget *toolbar, *scrollbar, *b_pause, *b_random;
  GtkTooltips *tooltips = NULL;

  tooltips = gtk_tooltips_new();
  _adjustment = gtk_adjustment_new(0.0, 0.0, 1.2, 0.1, 0.2, 0.2);
  scrollbar = gtk_hscrollbar_new(GTK_ADJUSTMENT(_adjustment));
  g_signal_connect(G_OBJECT(scrollbar), "button_press_event",
		   G_CALLBACK(_pressed_cb), NULL);
  g_signal_connect(G_OBJECT(scrollbar), "button_release_event",
		   G_CALLBACK(_released_cb), NULL);
  gtk_widget_show(scrollbar);
  b_pause = gtk_toggle_button_new_with_label("||");
  gtk_tooltips_set_tip(tooltips, b_pause, "Pause", NULL);
  g_signal_connect(G_OBJECT(b_pause), "toggled", G_CALLBACK(_pause_cb), NULL);
  gtk_widget_show(b_pause);
  b_random = gtk_toggle_button_new_with_label("R");
  gtk_tooltips_set_tip(tooltips, b_random, "Random", NULL);
  g_signal_connect(G_OBJECT(b_random), "toggled", G_CALLBACK(_random_cb),
		   NULL);
  gtk_widget_show(b_random);
  toolbar = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(toolbar), scrollbar, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(toolbar), b_pause, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(toolbar), b_random, FALSE, FALSE, 0);
  gtk_timeout_add(500, gui_update, NULL);
  return toolbar;
}

static void _destroy_cb(GtkWidget * widget, gpointer data)
{
  gtk_main_quit();
}

/* functions */
gboolean gui_update(gpointer data)
{
  int elapsed_time, total_time;
  char title[256];

  ogg_get_times(&elapsed_time, &total_time);
  sprintf(title, "Nogger %d:%02d [%d:%02d]", elapsed_time / 60,
	  elapsed_time % 60, total_time / 60, total_time % 60);
  gtk_window_set_title(GTK_WINDOW(_window), title);
  if (!_is_pressed)
    gtk_adjustment_set_value(GTK_ADJUSTMENT(_adjustment),
			     (double) elapsed_time / total_time);
  return TRUE;
}

int main(int argc, char *argv[])
{
  GtkWidget *tree, *toolbar, *vbox;

  srandom(time(NULL) + getpid());
  g_thread_init(NULL);
  gdk_threads_init();
  gtk_init(&argc, &argv);
  if (argc != 2) {
    fprintf(stdout, "\n Usage: nogger <path_to_files>\n\n");
    return 0;
  }
  tree = tree_create(argv[1]);
  if (tree == NULL) {
    fprintf(stderr, "Couldn't find any Ogg-files in `%s'\n", argv[1]);
    return 0;
  }
  gtk_widget_show(tree);
  toolbar = _toolbar_create();
  gtk_widget_show(toolbar);
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), tree, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  _window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_default_size(GTK_WINDOW(_window), -1, 250);
  gtk_container_add(GTK_CONTAINER(_window), vbox);
  g_signal_connect(G_OBJECT(_window), "destroy", G_CALLBACK(_destroy_cb),
		   NULL);
  gtk_widget_show(_window);
  gdk_threads_enter();
  gtk_main();
  gdk_threads_leave();
  return 0;
}
